@@ -238,6 +238,16 @@ impl Position {
         }
     }
 
+    /// Returns the position of the king with the given color.
+    pub fn find_king(&self, c: Color) -> Option<Square> {
+        let mut bb = &self.type_bb[PieceType::King.index()] & &self.color_bb[c.index()];
+        if bb.is_any() {
+            Some(bb.pop())
+        } else {
+            None
+        }
+    }
+
     /// Sets a piece at the given square.
     fn set_piece(&mut self, sq: Square, p: Option<Piece>) {
         self.board.set(sq, p);
@@ -262,15 +272,6 @@ impl Position {
         &bb & &self.move_candidates(sq, attack_pc.flip())
     }
 
-    fn find_king(&self, c: Color) -> Option<Square> {
-        let mut bb = &self.type_bb[PieceType::King.index()] & &self.color_bb[c.index()];
-        if bb.is_any() {
-            Some(bb.pop())
-        } else {
-            None
-        }
-    }
-
     fn log_position(&mut self) {
         // TODO: SFEN string is used to represent a state of position, but any transformation which uniquely distinguish positions can be used here.
         // Consider light-weight option if generating SFEN string for each move is time-consuming.
@@ -446,9 +447,7 @@ impl Position {
             if let Some(king_sq) = to.shift(0, if stm == Color::Black { -1 } else { 1 }) {
                 // Is the dropped pawn attacking the opponent's king?
                 if let Some(
-                    pc
-                    @
-                    Piece {
+                    pc @ Piece {
                         piece_type: PieceType::King,
                         ..
                     },
@@ -508,7 +507,8 @@ impl Position {
         Ok(MoveRecord::Drop { to, piece: pc })
     }
 
-    fn pinned_bb(&self, c: Color) -> Bitboard {
+    /// Returns a list of squares at which a piece of the given color is pinned.
+    pub fn pinned_bb(&self, c: Color) -> Bitboard {
         let ksq = self.find_king(c);
         if ksq.is_none() {
             return Bitboard::empty();
@@ -691,19 +691,19 @@ impl Position {
         // Build the initial position, all parts are required.
         parts
             .next()
-            .ok_or(SfenError {})
+            .ok_or(SfenError::MissingDataFields)
             .and_then(|s| self.parse_sfen_board(s))?;
         parts
             .next()
-            .ok_or(SfenError {})
+            .ok_or(SfenError::MissingDataFields)
             .and_then(|s| self.parse_sfen_stm(s))?;
         parts
             .next()
-            .ok_or(SfenError {})
+            .ok_or(SfenError::MissingDataFields)
             .and_then(|s| self.parse_sfen_hand(s))?;
         parts
             .next()
-            .ok_or(SfenError {})
+            .ok_or(SfenError::MissingDataFields)
             .and_then(|s| self.parse_sfen_ply(s))?;
 
         self.sfen_history.clear();
@@ -721,7 +721,7 @@ impl Position {
                         Err(_) => break,
                     }
                 } else {
-                    return Err(SfenError {});
+                    return Err(SfenError::IllegalMove);
                 }
             }
         }
@@ -761,7 +761,7 @@ impl Position {
 
         for (i, row) in rows.enumerate() {
             if i >= 9 {
-                return Err(SfenError {});
+                return Err(SfenError::IllegalBoardState);
             }
 
             let mut j = 0;
@@ -776,7 +776,7 @@ impl Position {
                         if let Some(n) = n.to_digit(10) {
                             for _ in 0..n {
                                 if j >= 9 {
-                                    return Err(SfenError {});
+                                    return Err(SfenError::IllegalBoardState);
                                 }
 
                                 let sq = Square::new(8 - j, i as u8).unwrap();
@@ -789,14 +789,14 @@ impl Position {
                     s => match Piece::from_sfen(s) {
                         Some(mut piece) => {
                             if j >= 9 {
-                                return Err(SfenError {});
+                                return Err(SfenError::IllegalBoardState);
                             }
 
                             if is_promoted {
                                 if let Some(promoted) = piece.piece_type.promote() {
                                     piece.piece_type = promoted;
                                 } else {
-                                    return Err(SfenError {});
+                                    return Err(SfenError::IllegalPieceType);
                                 }
                             }
 
@@ -809,7 +809,7 @@ impl Position {
 
                             is_promoted = false;
                         }
-                        None => return Err(SfenError {}),
+                        None => return Err(SfenError::IllegalPieceType),
                     },
                 }
             }
@@ -822,7 +822,7 @@ impl Position {
         self.side_to_move = match s {
             "b" => Color::Black,
             "w" => Color::White,
-            _ => return Err(SfenError {}),
+            _ => return Err(SfenError::IllegalSideToMove),
         };
         Ok(())
     }
@@ -846,7 +846,7 @@ impl Position {
                         Some(p) => self
                             .hand
                             .set(p, if num_pieces == 0 { 1 } else { num_pieces }),
-                        None => return Err(SfenError {}),
+                        None => return Err(SfenError::IllegalPieceType),
                     };
                     num_pieces = 0;
                 }
@@ -1057,6 +1057,37 @@ mod tests {
         }
     }
 
+    #[test]
+    fn find_king() {
+        setup();
+
+        let test_cases = [
+            (
+                "lnsgkgsnl/1r5b1/ppppppppp/9/9/9/PPPPPPPPP/1B5R1/LNSGKGSNL b - 1",
+                Some(SQ_5I),
+                Some(SQ_5A),
+            ),
+            ("9/3r5/9/9/6B2/9/9/9/3K5 b P 1", Some(SQ_6I), None),
+            (
+                "ln2r1knl/2gb1+Rg2/4Pp1p1/p1pp1sp1p/1N2pN1P1/2P2PP2/PP1G1S2R/1SG6/LK6L w 2PSp 1",
+                Some(SQ_8I),
+                Some(SQ_3A),
+            ),
+            (
+                "lnsg1gsnl/1r5b1/ppppppppp/9/9/9/PPPPPPPPP/1B5R1/LNSG1GSNL b - 1",
+                None,
+                None,
+            ),
+        ];
+
+        let mut pos = Position::new();
+        for case in test_cases.iter() {
+            pos.set_sfen(case.0).expect("failed to parse SFEN string");
+            assert_eq!(case.1, pos.find_king(Color::Black));
+            assert_eq!(case.2, pos.find_king(Color::White));
+        }
+    }
+
     #[test]
     fn player_bb() {
         setup();
